#!/bin/sh
#############################################################################
##
##  Copyright Compaq Computer Corp 1990, 1999, 2000. All rights reserved.
##
##  Restricted Rights: Use, duplication, or disclosure by the U.S.
##  Government is subject to restrictions as set forth in subparagraph
##  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
##  52.227-14 Alt. III, as applicable.
##
##  This software is proprietary to and embodies the confidential
##  technology of Digital Equipment Corporation. Possession, use, or
##  copying of this software and media is authorized only pursuant to a
##  valid written license from Digital or an authorized sublicensor.
##
##
##
##  NAME:	probe_linux
##
##  DESCRIP:	Looks at system and determines some interesting information
##              about it.
##
##  PARAMETERS:
##		P1: [gcc|nick|os|vendor|version|libc|init]
##                  Specify which piece of information we want about
##                  the system.
##
##
##  LOCATION:	This file is used in /usr/proj/decc2/bin
##
##  This file is under SCCS control in /usr/proj/decc2/bin.
##
##  Modifications:
##
##  10-Jan-2000	Kenneth Block
##	Initial design.
##
##  12-Jan-2000 Kenneth Block
##      Find GCC on RH6.0
##
##  17-Feb-2000 August G. Reinig
##      Added os as a supported parament, result is either Linux or UNIX.
##      Added support for Tru64 Unix.  Changes made has the makefile needs
##      to know which OS it is actually running on and it already uses
##      probe_linus.x to determine which flavor of linux is is running on
##
##  20-Mar-2000 Kenneth Block
##	Add support for Debian. Rewrote probing of libc to use dlopen.
##      Rewrote probe of gcc to use gcc -v.
##
##  23-Mar-2000 Kenneth Block
##	Always do the probe_dist so the results of that can be used in the
##	result of the tool. Do not do the libc probe is not Linux
##
##  17-Apr-2000 Kenneth Block
##      Add the ability to use this code as a library inside of another
##      script.
##
##  26-May-2000 Kenneth Block
##      Add code to test package version numbers.
##
##  07-Jul-2000 Kenneth Block
##      Add libcxx for getting the suffix for the cxx library.
##
##  12-Jul-2000 Kenneth Block
##      Add taso for determining if binutils supports taso
##
##   7-Aug-2000 Kenneth Block
##	Add Turbo Linux support
##
##   2-Oct-2000 Kenneth Block
##	Fixup dist probing
##
##   3-Oct-2000 Kenneth Block
##	Fixup debian
##
##   6-Oct-2000 Kenneth Block
##      Add probing of the binutils demangling
##
##  13-OCt-2000 Kenneth Block
##      Allow arbitrary length version numbers
##
##  13-Oct-2000 Kenneth Block
##      Fix problem for turbo linux
##

#
# Usage: installed_package_high_enough package minimum_version 
#
# Check to see if installed package version is high enough
#

#
# cut_rest gives you the rest of the list, or nothing
# if no more
#
cut_rest()
{
  rest=`echo $1 | cut -f2- -d$2`
  if [ "$rest" = "$1" -o "$rest" = "" ]; then
    echo "0"
  else
    echo "$rest"
  fi
}

version_high_enough()
{
# usage: minimum_version actual_version
#
#        return 0 means version number is high enough

  min_num=`echo $1 | cut -f1 -d-`
  min_spin=`cut_rest $1 -`

  act_num=`echo $2 | cut -f1 -d-`
  act_spin=`cut_rest $2 -`

# spin number is only going to be interesting if nums the same
  if [ "$min_num" = "$act_num" ]; then
    if [ $min_spin -le $act_spin ]; then
      return 0
    else
      return 1
    fi
  fi


  while (true) do
    min=`echo $min_num | cut -f1 -d.`
    act=`echo $act_num | cut -f1 -d.`

    if [ $min -lt $act ]; then
      return 0
    elif [ $min -gt $act ]; then
      return 1
    fi

    min_num=`cut_rest $min_num .`
    act_num=`cut_rest $act_num .`

    if [ "$act_num" = "0" -a "$min_num" = "0" ]; then
       return 0
    fi
  done
}

#
# Returns 0 is package is high enough
#         1 for unknown error
#         2 package not installed
#         3 package version is not high enough
#

package_high_enough()
{
  package="$1"
  package_file="$2"
  min_ver="$3"

  if [ ! -r $package_file ]; then
  # package must not be installed
    return 2
  fi

  #
  # Determine the version by slicing up the symlink from /usr/lib
  # which should look something like:
  #
  # %file /usr/lib/libcpml.so     
  # /usr/lib/libcpml.so: symbolic link to ./compaq/cpml-5.0.0/libcpml_ev5.so
  # 
  symlink=`file $package_file | sed -e 's|[^ ]*: symbolic link to ||'`
  package_ver_name=`echo $symlink | sed -e "s|.*/\($package-[0-9.]*\)/.*|\1|"`
  package_ver=`echo $package_ver_name | cut -f2 -d-`

  version_high_enough $min_ver $package_ver

  if [ $? -eq 0 ]; then
     # package version is high enough
     return 0
  fi

  # Not high enough
  return 3
}


probe_dist()
{
  DIST_OS="UNKNOWN"
  DIST_VENDOR="UNKNOWN"

  DIST_FILES=""

  for i in /etc/*-release /etc/debian_version; do
    if [ -r $i ]; then
      DIST_FILES="$DIST_FILES $i"
    fi
  done

  if [ ! "$DIST_FILES" = "" ]; then
     DIST_OS="Linux"
     if [ -r /etc/kondara-release ]; then
       DIST_VENDOR="kondara"
     elif [ -r /etc/redhat-release ]; then
       DIST_VENDOR="rh"
     elif [ -r /etc/SuSE-release ]; then
       DIST_VENDOR="suse"
     elif [ -r /etc/turbolinux-release ]; then
       DIST_VENDOR="tl"
     elif [ -r /etc/debian_version ]; then
       DIST_VENDOR="debian"
     fi
  elif [ -x /usr/sbin/sizer ]; then
    # This script was only intended to be run on Linux
    # This cruft was added for cases were it was unintentionally
    # run on Tru64
    DIST_OS="Tru64_UNIX"
    DIST_VENDOR="compaq"
  fi
}

#
# The versions are only to be used internal to the build environment
# due to the fragile nature of the procedure.
#
probe_version()
{
  if [ "$DIST_OS" = "Tru64_UNIX" ]; then
    # This script was only intended to be run on Linux
    # This cruft was added for cases were it was unintentionally
    # run on Tru64
    DIST_VERSION=`/usr/sbin/sizer -v | sed -e 's/^.*\(V[^ ]*\).*/\1/'`
  elif [ ! "DIST_FILES" = "" ]; then
    if [ "$DIST_VENDOR" = "kondara" ]; then
      ver_line=`cat /etc/kondara-release | grep Kondara`
    elif [ "$DIST_VENDOR" = "suse" ]; then
      ver_line=`cat /etc/SuSE-release | grep "SuSE Linux"`
    else
      ver_line=`cat $DIST_FILES`
    fi
  
    DIST_VERSION=`echo $ver_line | sed -e 's|[^0-9.]*||g'`
  else
    DIST_VERSION="UNKNOWN"
  fi
} 

probe_gcc()
{
    specs=`gcc -v 2>&1 | grep specs | sed -e 's|Reading specs from ||g'`

    if [ ! -r $specs ]; then
      GCC_PATH="Unknown GCC"
    else
      GCC_PATH=`dirname $specs`
    fi
}

probe_libc()
{
  if [ "$DIST_OS" = "Linux" ]; then
    cat > /tmp/libc$$.c <<EOF
#include <stdio.h>
#include <dlfcn.h>

int main()
{
  void *my_handle;
  char * (*get_version)();
  char *version;

  my_handle = dlopen(NULL,RTLD_LAZY);
  if (my_handle == NULL) {
    printf("dlopen: %s\n",dlerror());
    return 1;
  }

  /* gnu_get_libc_version seems to exist for 6.0 or higher */
  get_version=dlsym(my_handle,"gnu_get_libc_version");
  if (get_version != NULL) {
    printf("%s\n",get_version());
    return 0;
  }

  /* for pre-6.0 */
  version=dlsym(my_handle,"__libc_version");
  if (version != NULL) {
    printf("%s\n",version);
    return 0;
  }

  printf("unknown\n");

  return 0;
}
EOF
    cc -o /tmp/libc$$ /tmp/libc$$.c -ldl
    LIBC=`/tmp/libc$$`
    rm /tmp/libc$$ /tmp/libc$$.c
  else
    # How to be determine libc version of Tru64, does it matter?
    LIBC="UNKNOWN"
  fi
}

probe_libcxx()
{
#
#   Probing libc can take a while. Only do it once.
#
    if [ "$LIBC" = "" ]; then
      probe_libc
    fi
    if [ "`echo $LIBC | cut -f1,2 -d.`" = "2.0" ]; then
      LIBCXX="rh52"
    else
      LIBCXX="rh60"
    fi
}

probe_taso()
{
#
# TASO will return "YES", "NO", or "an error string"
#
  TASO="YES"

  if [ "$DIST_OS" = "Linux" ]; then
    # Does ld claim to support taso?
    if [ "`ld --help | grep taso`" = "" ]; then
       TASO="NO"
       return
    fi

    # Verify taso works 
    cat > /tmp/tasolib$$.c <<EOF
#include <stdio.h>
#include <stdlib.h>

#define pointer_not_taso(x) (((unsigned long)x) >= 0x100000000L)

int x;

#ifdef __cplusplus
extern "C"
#endif

int f()
{
  int err = 0;
  int *y = (int*)malloc(32);

  if (pointer_not_taso(&x)) {
    printf("Global variable X is at %p\n",&x);
    err=2;
  }

  if (pointer_not_taso(&f)) {
    printf("Function f is at %p\n",&f);
    err=2;
  }

  if (pointer_not_taso(y)) {
    printf("Malloc is at %p\n",y);
    err=2;
  }
  return err;
}
EOF
    cat > /tmp/tasomain$$.c <<EOF
#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#ifndef LIBNAME
#define LIBNAME "libt.so"
#endif

extern 
#ifdef __cplusplus
"C"
#endif
int f2();

typedef int (*func_t)();

int main()
{
  void *handle;
  func_t f;
  int err=0;

  if ((handle=dlopen(LIBNAME,RTLD_LAZY))==NULL) {
    printf("%s\n",dlerror());
    exit(1);
  }

  if ((f=(func_t)dlsym(handle,"f"))==NULL) {
    printf("%s\n",dlerror());
    exit(1);
  }
  
  err += f();
  err += f2();

  return err;
}
EOF
    cc -shared -o /tmp/libtaso$$.so /tmp/tasolib$$.c \
       > /tmp/tasomain$$.out 2>&1
    ret=$?
    if [ "`cat /tmp/tasomain$$.out`" != "" ]; then
      ret=1
    fi

    if [ $ret -eq 0 ]; then
      cc -shared -o /tmp/libtaso2$$.so /tmp/tasolib$$.c -Df=f2 \
         > /tmp/tasomain$$.out 2>&1
      ret=$?
      if [ "`cat /tmp/tasomain$$.out`" != "" ]; then
        ret=1
      fi
    fi

    if [ $ret -eq 0 ]; then
      sleep 2
      cc -o /tmp/tasomain$$ /tmp/tasomain$$.c -DLIBNAME=\"/tmp/libtaso$$.so\" \
          -Wl,-taso -Wl,-rpath,/tmp -L/tmp -ltaso2$$ -ldl \
          > /tmp/tasomain$$.out 2>&1
      ret=$?
      if [ "`cat /tmp/tasomain$$.out`" != "" ]; then
        ret=1
      fi
    fi

    if [ $ret -eq 0 ]; then
      /tmp/tasomain$$ > /tmp/tasomain$$.out 2>&1
      ret=$?
      if [ $ret -gt 1 ]; then
        TASO="NO"
        ret=0
      fi
    fi

    if [ $ret -ne 0 ]; then
       TASO="/tmp/tasomain$$.out"
    else
       rm -f /tmp/tasomain$$.out
    fi 

    rm -f /tmp/libtaso$$.so /tmp/libtaso$$.so /tmp/tasolib$$.c
    rm -f /tmp/tasomain$$ /tmp/tasomain$$.c
  fi
}

probe_demangle()
{
  CXX_LOG="/tmp/cxx_install$$.log"

  if [ "`ld --help 2>&1 | grep '\--demangle' | grep STYLE`" != "" ]; then
    DEMANGLE="demangle"
  else
    # Assume style by default in case early versions did not 
    # have documentation
    DEMANGLE="style"
  fi

    cat > /tmp/demangle$$.c <<EOF
int main() { return 0; }
EOF

  cc -o /tmp/demangle$$ /tmp/demangle$$.c -Wl,--$DEMANGLE=compaq > $CXX_LOG 2>&1
  if [ ! "`grep -i 'ld:' $CXX_LOG | grep -i 'cannot open compaq'`" = "" ]; then
    DEMANGLE="unsupported"

  elif [ ! "`grep unrecognized $CXX_LOG | grep style`" = "" ]; then
    DEMANGLE="unsupported"

  elif [ ! "`grep -i 'ld:' $CXX_LOG | grep style | grep allow`" = "" ]; then
    # regression introduced causing --style not to take an argument
    DEMANGLE="unsupported"

  elif [ ! "`cat $CXX_LOG`" = "" ]; then
    DEMANGLE="$CXX_LOG"

  elif [ ! -x /tmp/demangle$$ ]; then
    DEMANGLE="Unexpected error: /tmp/demangle$$ is not executable"

  else
    /tmp/demangle$$ > $CXX_LOG 2>&1
    if [ ! "$?" = "0" ]; then
      DEMANGLE="$CXX_LOG"
    else
      rm -f $CXX_LOG /tmp/demangle$$ /tmp/demangle$$.c /tmp/demangle$$.c
    fi
  fi
}

probe_dist

case "$1" in
"init")
#   Do nothing, this code is being included inside another script
#   which will call the functions as needed
    ;;
"os" )
    echo $DIST_OS
    ;;

#
# nick is only intended to be used in the build
#
"nick" )
    probe_version
    echo "$DIST_VENDOR$DIST_VERSION" | sed -e 's|\.||g'
    ;;

"vendor" )
    echo $DIST_VENDOR
    ;;

#
# version is only intended to be used in the build
#
"version" )
    probe_version
    echo $DIST_VERSION
    ;;

"gcc" )
    probe_gcc
    echo $GCC_PATH
    ;;

"libc" )
    probe_libc
    echo $LIBC
    ;;

"libcxx" )
    probe_libcxx
    echo $LIBCXX
    ;;

"taso" )
    probe_taso
    echo $TASO
    ;;

"demangle" )
    probe_demangle
    echo $DEMANGLE
    ;;

"package_high_enough" )
    shift
    package_high_enough $*
    status=$?

    if [ $status -eq 0 ]; then
       echo "Package version is high enough"
    elif [ $status -eq 2 ]; then
       echo "Package is not installed"
    elif [ $status -eq 3 ]; then
       echo "Package version is not high enough"
    else
       echo "Unknown error"
       cat /tmp/test_package_version$$
    fi
    ;;

*)
    cat <<EOF
Usage: `basename $0` [gcc|nick|os|vendor|version|libc|libcxx|taso|demangle]
EOF
    exit 1
esac
