#!/bin/sh
#
#  *****************************************************************
#  *   Copyright 2000 Compaq Computer Corporation                  *
#  *                                                               *
#  *   COMPAQ Registered in U.S. Patent and Trademark Office.      *
#  *                                                               *
#  *   Confidential computer software. Valid license from Compaq   *
#  *   or authorized sublicensor required for possession, use or   *
#  *   copying. Consistent with FAR 12.211 and 12.212, Commercial  *
#  *   Computer Software, Computer Software Documentation, and     *
#  *   Technical Data for Commercial Items are licensed to the     *
#  *   U.S. Government under vendor's standard commercial license. *
#  *****************************************************************
#
# HISTORY
#
# @(#)$RCSfile: protect_headers_setup.sh,v $ $Revision: 1.4 $ (DEC) $Date: 1999/12/03 04:25:53 $
#
# NAME
#	protect_headers_setup.sh
#
# SYNOPSIS
#	protect_headers_setup.sh [-l|-d|-i|-v] [-f] [dirname...]
#
# DESCRIPTION
#
#	This script can be used to "protect" library header files when
#	compiling with -Zpn (where n is not equal to
#	8).  You must protect header files so that data layouts
#	compiled into an application when the
#	header file is included into a compilation unit using the
#	#include directive are the same as those made when the library
#	code associated with the header file was compiled.
#
#	The system header files are not protected by default.  Thus, most
#	programs will not work correctly when compiled with
# 	-Zpn (where n is not equal to 8), unless you
#	protect the system header files and any other header files
#	associated with code that assumes the
#	default natural member alignment.
#
#	This script creates symbolic links to special header prologue
#	and epilogue files in all directories in a directory tree that
#	contain header (.h) files.  The default directory tree root is
#	/usr/include and all directories named in -SD options contained
#	in the comp.config file stored in the same directory as this
#	script - but other roots can be specified.
#
#	The special header files are named __DECC_include_prologue.h
#	and __DECC_include_epilogue.h.  The compiler checks for files
#	with these special names when processing #includes.  If the
#	special prologue file exists in the same directory as a
#	#included file, the contents of the prologue file are
#	processed just before the #included file.  Similarly, if the
#	epilogue file exists in the same directory as the #included
#	file, it is processed just after the #included file.
#
#	See the protect_headers_setup(8) and ccc(1) man pages for
#	more information.
#
# OPTIONS
#
#	If dirname... is/are specified, use those as roots, otherwise use
#	/usr/include and all directories named in -SD options contained
#       in the comp.config file stored in the same directory as this
#       script as roots.
#
#       The command options are meant to match their counterparts in
#	the Tru64 UNIX setld command in functionality.
#
#	-l	Loads the protected header file links
#		This will not overwrite existing files or links
#
#	-d	Deletes the protected header file links only
#
#	-f	Force delete of protected header links and files
#
#	-i	Inventories the directories where protected
#		header files or links are installed
#
#	-v	Verifies the directories and lists subdirectories
#		that do not contain the protected header file links.
#
# NOTES
#
#  When adding links, any preexisting files or links with these files
#  are not replaced.
#
#  When removing links, real files are removed if -f is specified,
#  but not if -d is specified.
#
#  Optional switches may not be combined.
#
# ********************************************************************


USAGE="Usage:  $0 [-l|-d|-i|-v] [-f] [dirname...]"

HOST_DIR=/usr/lib/compaq/ccc-shared
CONFIG_DIR=`dirname $0`

PROLOG=__DECC_include_prologue.h
EPILOG=__DECC_include_epilogue.h

TMPFILE1=/tmp/_protect_tmp1_$$
TMPFILE2=/tmp/_protect_tmp2_$$

OPTION=FALSE
FORCE=

case $1 in
   -l)	OPTION=LINK
	;;
   -lf)	OPTION=LINK
	FORCE=-f
	;;
   -d)	OPTION=DELETE
	;;
   -df)	OPTION=DELETE
	FORCE=-f
	;;
   -f)	echo "ERROR: -f must follow -l or -d option"
	echo $USAGE; exit 1
	;;
   -i)	OPTION=INVENTORY
	;;
   -v)	OPTION=VERIFY
	;;
   "")	echo "ERROR: Must specify an option"
	echo $USAGE; exit 1
	;;
   *)	echo "ERROR: Unknown option: $1"
	echo $USAGE; exit 1
	;;
esac

shift

if [ "$1" = "-f" ]; then
   FORCE=-f
   shift
fi

umask 22
if [ "$OPTION" = "LINK" ]; then

   if [ ! -d $HOST_DIR ]; then
      mkdir -p $HOST_DIR
   fi

   if [ ! -f $HOST_DIR/$PROLOG ]; then
      echo "Creating $HOST_DIR/$PROLOG"
      cat << EOPROLOG > $HOST_DIR/$PROLOG
#ifdef __PRAGMA_ENVIRONMENT
#pragma __environment save
#pragma __environment header_defaults
#ifdef __DECC_EMPTY_SD_OPTION
#pragma message restore  /* undo the message part of environment */
#pragma message save     /* make the message stack the right depth */
#endif
#else
#error "__DECC_include_prologue.h:  This compiler does not support pragma __environment"
#endif
EOPROLOG
   fi

   if [ ! -f $HOST_DIR/$EPILOG ]; then
      echo "Creating $HOST_DIR/$EPILOG"
      cat << EOEPILOG > $HOST_DIR/$EPILOG
#ifdef __PRAGMA_ENVIRONMENT
#pragma __environment restore
#else
#error "__DECC_include_epilogue.h:  This compiler does not support pragma __environment"
#endif
EOEPILOG
   fi
fi

DIR="$*"
if [ -z "$DIR" ] ; then
    echo "No directories specified, getting defaults from comp.config."
    if [ ! -r $CONFIG_DIR/comp.config ] ; then
	echo "ERROR: No existing readable file $CONFIG_DIR/comp.config."
	exit 1
    fi
    DIR="/usr/include `tr -s ' 	' '\012' < $CONFIG_DIR/comp.config \
                       | sed -n -e 's/^-SD//p'`"
else
    for dir in $DIR ; do
	if [ ! -d $dir ] ; then
	    echo "WARNING: Argument '$dir' is not a directory, ignored."
	fi
    done
fi

echo "Running $OPTION $FORCE option on $DIR ..."

trap "rm -f $TMPFILE1 $TMPFILE2; exit" 1 2 15
rm -f $TMPFILE1 $TMPFILE2

echo "Generating list of reachable directories ..."
#
# Get list of directories "reachable" from $DIR directories.
# Reachable means either below it in the directory tree, or pointed-to
# by a symbolic link below it in the directory tree.  The -follow
# option to GNU find seems to make this quite trivial:
#

find $DIR -follow -type d -print > $TMPFILE1 2>/dev/null

sed 's%\/*$%%g' $TMPFILE1 | sort -u > $TMPFILE2

echo "done."

#
# Go thru the list of directories and link/delete/verify
#
for dir in `cat $TMPFILE2`
do
   ls $dir/*.h 1> /dev/null 2>&1
   if [ $? = 0 ]; then
      if [ "$OPTION" = "DELETE" ]; then
	 if [ -L $dir/$PROLOG ]; then
 	    echo "Removing $dir/$PROLOG"
            rm $dir/$PROLOG               
	 elif [ -f $dir/$PROLOG ]; then
            if [ "$FORCE" = "-f" ]; then
    	       echo "Removing $dir/$PROLOG"
               rm $dir/$PROLOG               
            else
               echo "Real file $dir/$PROLOG not removed"
            fi
	 else
	    echo "No prolog file found in $dir"
         fi
         if [ -L $dir/$EPILOG ]; then
 	    echo "Removing $dir/$EPILOG"
	    rm $dir/$EPILOG
	 elif [ -f $dir/$EPILOG ]; then
            if [ "$FORCE" = "-f" ]; then
 	       echo "Removing $dir/$EPILOG"
	       rm $dir/$EPILOG
            else
 	       echo "Real file $dir/$EPILOG not removed"
            fi
	 else
	    echo "No epilog file found in $dir"
	 fi
      elif [ "$OPTION" = "VERIFY" ]; then
	 if [ ! -f $dir/$PROLOG ] && [ ! -f $dir/$EPILOG ]; then
	    echo "Warning: No protected header files found in $dir"
	 elif [ ! -f $dir/$PROLOG ]; then
	    echo "Warning: No $PROLOG found in $dir"
	 elif [ ! -f $dir/$EPILOG ]; then
	    echo "Warning: No $EPILOG found in $dir"
	 fi
      elif [ "$OPTION" = "INVENTORY" ]; then
	 if [ -L $dir/$PROLOG ]; then
	    if [ -s $dir/$PROLOG ]; then
	       echo "Link verified: $dir/$PROLOG"
	    else
	       echo "Invalid link : $dir/$PROLOG"
	    fi
	 elif [ -f $dir/$PROLOG ]; then
 	    echo "File found   : $dir/$PROLOG"
	 else
	    echo "No prolog file found in $dir"
         fi
         if [ -L $dir/$EPILOG ]; then
            if [ -s $dir/$EPILOG ]; then
 	       echo "Link verified: $dir/$EPILOG"
            else
 	       echo "Invalid link : $dir/$EPILOG"
            fi
	 elif [ -f $dir/$EPILOG ]; then
 	    echo "File found   : $dir/$EPILOG"
	 else
	    echo "No epilog file found in $dir"
	 fi
      elif [ "$OPTION" = "LINK" ]; then
	 if [ ! -f $dir/$PROLOG ] || [ "$FORCE" = "-f" ]; then
	    echo "Linking $dir/$PROLOG"
            ln -snf $HOST_DIR/$PROLOG $dir
         else
	    echo "File exists: $dir/$PROLOG, not linked"
	 fi
	 if [ ! -f $dir/$EPILOG ] || [ "$FORCE" = "-f" ]; then
            echo "Linking $dir/$EPILOG"
	    ln -snf $HOST_DIR/$EPILOG $dir
         else
	    echo "File exists: $dir/$PROLOG, not linked"
	 fi
      fi
   fi
done

# Cleanup
rm -f $TMPFILE1 $TMPFILE2

