#ifndef		__COMPLEX_HXX
#define		__COMPLEX_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Complex arithmetic
//
//  MODULE:	complex
//
//  INFORMATION HIDDEN:
//
//	Representation of a complex number.
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of complex numbers and 
//	defines the functions which operate on them.
//
//	Differences from AT&T's product:
//	    Add tan(const complex &).
//	    Add tanh(const complex &).
//	    Add arg1(const complex &).
//	    Add sqr(const complex &).
//	    Make the data members of c_exception be public instead of private.
//
//  DESIGN ISSUES:
//
//	The representation is kept in cartesian coordinates (as in FORTRAN)
//	to prevent errors resulting from converting back and forth between
//	coordinate systems (including altering the parameters to the operators,
//	which are passed by reference).
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>
#include <iostream.hxx>

extern "C"
{
#include <stddef.h>

#if ! defined(complex) && ! defined(__NT)
    // in case <math.h> defines its own struct complex, give it a different
    // name; <math.h> on Windows NT (tm) defines its own complex macro
#   define complex __cxxl_math_complex
#endif
#include <math.h>
#undef complex

}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif


// ANSI C says that these may be macros in <math.h> but it prevents overloading
#undef sin
#undef sinh
#undef cos
#undef cosh
#undef exp
#undef pow
#undef sqrt



//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif


class complex

//++
//  CLASS DESCRIPTION:
//
//	Complex number (real part plus imaginary part)
//
//  DESIGN:
//
//	The complex number is only stored in cartesian coordinates to prevent
//	errors resulting from converting back and forth.  ("Floating point
//	numbers are like little piles of sand; every time you move one, you
//	lose a little sand and pick up a little dirt.")
//
//	The operators are defined as friends so that implicit conversions can
//	be done, such as treating 1+complex(3,4) as complex(1)+complex(3,4).
//	Otherwise, the operators should really be defined as member functions
//	instead of friend functions.
//
//	Parameters are passed by reference.
//--

{
    inline friend complex polar(double, double = 0);
    friend double abs(const complex &);
    friend double norm(const complex &);
    friend double arg(const complex &);
    friend double arg1(const complex &);
    inline friend complex conj(const complex &);
    friend complex sin(const complex &);
    friend complex sinh(const complex &);
    friend complex cos(const complex &);
    friend complex cosh(const complex &);
    friend complex tan(const complex &);
    friend complex tanh(const complex &);
    inline friend double imag(const complex &);
    inline friend double real(const complex &);
    friend complex log(const complex &);
    friend complex exp(const complex &);
    friend complex pow(double, const complex &);
    friend complex pow(const complex &, int);
    friend complex pow(const complex &, double);
    friend complex pow(const complex &, const complex &);
    friend complex sqrt(const complex &);
    friend complex sqr(const complex &);
    inline friend complex operator-(const complex &);
    inline friend complex operator+(const complex &, const complex &);
    inline friend complex operator-(const complex &, const complex &);
    friend complex operator*(const complex &, const complex &);
    friend complex operator/(const complex &, const complex &);
    inline friend int operator==(const complex &, const complex &);
    inline friend int operator!=(const complex &, const complex &);
    friend ostream &operator<<(ostream &, const complex &);
    friend istream &operator>>(istream &, complex &);

private:
    double cxxl_re;			// Real part.
    double cxxl_im;			// Imaginary part.

    complex cxxl_exp(int &) const;
    complex cxxl_real_overflow_return_value() const;

public:

    inline complex(double, double = 0);
    inline complex();

    inline complex & operator-=(const complex &);
    inline complex & operator+=(const complex &);
    complex & operator*=(const complex &);
    complex & operator/=(const complex &);
};



//
//
//  EXTERNAL DATA
//
//

static const complex complex_zero(0,0);

inline complex::complex(
    double x,			// initial real part
    double y			// initial imaginary part, defaults to 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This is a constructor to create & initialize the complex number from
//	cartesian coordinates.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Memory is allocated
//
//  POSTCONDITIONS:
//
//	cxxl_re == x
//	cxxl_im == y
//--

    :
    cxxl_re(x),
    cxxl_im(y)
{
}

inline complex::complex(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	This is a constructor to create & initialize the complex number to 0.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Memory is allocated
//
//  POSTCONDITIONS:
//
//	*this == 0
//--

    :
    cxxl_re(0),
    cxxl_im(0)
{
}

inline complex polar(
    double rho,				// initial magnitude
    double theta			// initial angle, defaults to 0
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Create a complex number and initialize it in polar coordinates.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	A new complex number.
//
//  POSTCONDITIONS:
//
//	abs() == rho
//	sin(arg()) == sin(theta)
//	cos(arg()) == cos(theta)
//--

{
    return complex(rho * cos(theta), rho * sin(theta));
}

inline double real(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Return the real part of a complex number.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	The real part of z1
//--

{
    return z1.cxxl_re;
}

inline double imag(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Return the imaginary part of a complex number.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	The imaginary part of z1
//--

{
    return z1.cxxl_im;
}

inline complex conj(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Take the conjugate of a complex number.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	The conjugate of z1
//
//  DESIGN:
//
//	conj((re,im)) = (re, -im)
//--

{
    return complex(z1.cxxl_re, - z1.cxxl_im);
}

inline complex operator-(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Unary negation of a complex number.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	-z1
//
//  DESIGN:
//
//	-(re,im) = (-re,-im)
//--

{
    return complex(- z1.cxxl_re, - z1.cxxl_im);
}

inline complex operator+(
    const complex &z1,			// A complex number
    const complex &z2			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Sum of complex numbers.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	z1+z2
//
//  DESIGN:
//
//	(re1,im1)+(re2,im2) = (re1+re2,im1+im2)
//--

{
    return complex(z1.cxxl_re + z2.cxxl_re, z1.cxxl_im + z2.cxxl_im);
}

inline complex operator-(
    const complex &z1,			// A complex number
    const complex &z2			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Difference of complex numbers.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	z1-z2
//
//  DESIGN:
//
//	(re1,im1)-(re2,im2) = (re1-re2,im1-im2)
//--

{
    return complex(z1.cxxl_re - z2.cxxl_re, z1.cxxl_im - z2.cxxl_im);
}

inline complex & complex::operator+=(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Add a complex number to a complex number.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	*this is increased by z1
//
//  DESIGN:
//
//	(re1,im1)+(re2,im2) = (re1+re2,im1+im2)
//--

{
    cxxl_re += z1.cxxl_re;
    cxxl_im += z1.cxxl_im;

    return *this;
}

inline complex & complex::operator-=(
    const complex &z1			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Subtract a complex number from a complex number.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	*this is decreased by z1
//
//  DESIGN:
//
//	(re1,im1)-(re2,im2) = (re1-re2,im1-im2)
//--

{
    cxxl_re -= z1.cxxl_re;
    cxxl_im -= z1.cxxl_im;

    return *this;
}

inline int operator==(
    const complex &z1,			// A complex number
    const complex &z2			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Compare complex numbers for equality.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	If z1 == z1, a non-0 value
//	If z1 != z1, 0
//--

{
    return (z1.cxxl_re == z2.cxxl_re && z1.cxxl_im == z2.cxxl_im);
}

inline int operator!=(
    const complex &z1,			// A complex number
    const complex &z2			// A complex number
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Compare complex numbers for inequality.
//
//  FRIEND OF:	complex
//
//  RETURN VALUE:
//
//	If z1 != z1, a non-0 value
//	If z1 == z1, 0
//--

{
    return (z1.cxxl_re != z2.cxxl_re || z1.cxxl_im != z2.cxxl_im);
}

class c_exception

//++
//  CLASS DESCRIPTION:
//
//	Complex arithmetic exception information (function, parameters, error
//	type, and default return value)
//
//  DESIGN:
//
//	Note that the private data members of this class are really part of
//	the public interface, because they may be used by the optionally-user-
//	supplied friend function, complex_error(c_exception &).  (This is why
//	they do not have the facility prefix like other private members of a
//	class.)  Also note that they are public instead of private for access by
//	complex_error(c_exception &)-like functions which are established with
//	set_complex_error(cxxl_p_complex_error_t) instead of by order of
//	linking getting the "correct" instance of complex_error(c_exception &).
//
//	See also:
//
//	    "ULTRIX-32 Programmer's Manual Sections 2, 3, 4, and 5
//	    (AA-BG54E-TE)" page 3-166ff on subroutine "matherr ( 3m )"
//	    for information on the analogous capability in the ULTRIX C library.
//--

{
    friend complex exp(const complex &);
    friend complex sinh(const complex &);
    friend complex cosh(const complex &);
    friend complex log(const complex &);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    friend int complex_error(c_exception &);


public:

    int type;				// Error type (SING, OVERFLOW,
					// or UNDERFLOW).
    char *name;				// The name of the function that
					// incurred the error.
    complex arg1;			// Left argument of the function.
    complex arg2;			// Right argument of the function.
    complex retval;			// Value to be returned by
					// the function; may be changed by
					// complex_error(c_exception &).

private:

    void invoke_complex_error_handling(int n_args = 1);

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

public:

    c_exception(char *function_name, const complex &function_arg1,
	const complex &function_arg2 = complex_zero);
};



//
//
//  CONSTANTS
//
//

// ULTRIX puts these error codes in <math.h> but VMS doesn't

#ifndef SING
#define SING 2
#endif

#ifndef OVERFLOW
#define OVERFLOW 3
#endif

#ifndef UNDERFLOW
#define UNDERFLOW 4
#endif

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif

// End of complex.hxx
