#ifndef		__GENERIC_HXX
#define		__GENERIC_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Generic data structures (primordial parameterized classes)
//
//  MODULE:	generic
//
//  MODULE DESCRIPTION:
//
//	This module provides a generic/parameterized type facility which uses
//	the macro preprocessor instead of C++ templates.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
//  TYPE DEFINITIONS
//
//

// pointer to a generic error handling function
typedef int (*GPT)(int, char *);



//
//
//  EXTERNAL FUNCTIONS
//
//

// the default error handling function
extern int genericerror(int, char *);

//++
//  MACRO NAME:	name2
//
//  FUNCTIONAL DESCRIPTION:
//
//	Concatenate identifier (segments) during compilation preprocessing to
//	form a new identifier.
//
//  FORMAL PARAMETERS:
//
//	S1: Segment 1 of the concatenated identifier
//	S2: Segment 2 of the concatenated identifier
//
//  EXPANSION VALUE:
//
//	The identifier formed by the concatenation of the actual parameters.
//
//  DESIGN:
//
//	The ## operator is used to concatenate identifier segments.
//	It is invoked in a sub-macro to allow parameter expansion to work.
//
//--

#define name2(S1, S2)	cxxl_name2(S1, S2)
#define cxxl_name2(S1, S2)	S1##S2

//++
//  MACRO NAME:	name3
//
//  FUNCTIONAL DESCRIPTION:
//
//	Concatenate identifier (segments) during compilation preprocessing to
//	form a new identifier.
//
//  FORMAL PARAMETERS:
//
//	S1: Segment 1 of the concatenated identifier
//	S2: Segment 2 of the concatenated identifier
//	S3: Segment 3 of the concatenated identifier
//
//  EXPANSION VALUE:
//
//	The identifier formed by the concatenation of the actual parameters.
//
//  DESIGN:
//
//	The ## operator is used to concatenate identifier segments.
//	It is invoked in a sub-macro to allow parameter expansion to work.
//
//--

#define name3(S1, S2, S3)	cxxl_name3(S1, S2, S3)
#define cxxl_name3(S1, S2, S3)	S1##S2##S3

//++
//  MACRO NAME:	name4
//
//  FUNCTIONAL DESCRIPTION:
//
//	Concatenate identifier (segments) during compilation preprocessing to
//	form a new identifier.
//
//  FORMAL PARAMETERS:
//
//	S1: Segment 1 of the concatenated identifier
//	S2: Segment 2 of the concatenated identifier
//	S3: Segment 3 of the concatenated identifier
//	S4: Segment 4 of the concatenated identifier
//
//  EXPANSION VALUE:
//
//	The identifier formed by the concatenation of the actual parameters.
//
//  DESIGN:
//
//	The ## operator is used to concatenate identifier segments.
//	It is invoked in a sub-macro to allow parameter expansion to work.
//
//--

#define name4(S1, S2, S3, S4)	cxxl_name4(S1, S2, S3, S4)
#define cxxl_name4(S1, S2, S3, S4)	S1##S2##S3##S4

//++
//  MACRO NAME:	declare
//
//  FUNCTIONAL DESCRIPTION:
//
//	Declare a generic (parameterized) class for a given type.
//	This also defines the inline member functions of the class.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T:
//	    The type parameter to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//
//  IMPLICIT INPUT PARAMETERS:
//
//	A macro with the name of the generic class name followed by the word
//	"declare".  (e.g. vectordeclare)
//
//  EXPANSION VALUE:
//
//	Invocation of the user-defined macro to declare the class.
//	(See implicit input parameters.)
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Declares the class (and possibly functions) which are declared by the
//	user-defined macro.
//
//  DESIGN:
//
//	A sub-macro is used to allow parameter expansion to work.
//
//--

#define declare(CLASS, T) cxxl_declare(CLASS, T)
#define cxxl_declare(CLASS, T) name2(CLASS, declare)(T)

//++
//  MACRO NAME:	implement
//
//  FUNCTIONAL DESCRIPTION:
//
//	Implement a generic (parameterized) class for a given type.
//	This defines the non-inline member functions of the class.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T:
//	    The type parameter to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//
//  IMPLICIT INPUT PARAMETERS:
//
//	A macro with the name of the generic class name followed by the word
//	"implement".  (e.g. vectorimplement)
//
//  EXPANSION VALUE:
//
//	Invocation of the user-defined macro to define the non-inline
//	member functions of the class.
//	(See implicit input parameters.)
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Defines the functions which are defined by the user-defined macro.
//
//  DESIGN:
//
//	A sub-macro is used to allow parameter expansion to work.
//
//--

#define implement(CLASS, T) cxxl_implement(CLASS, T)
#define cxxl_implement(CLASS, T) name2(CLASS, implement)(T)

//++
//  MACRO NAME:	declare2
//
//  FUNCTIONAL DESCRIPTION:
//
//	Declare a generic (parameterized) class for 2 given types.
//	This also defines the inline member functions of the class.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T1, T2:
//	    The type parameters to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//
//  IMPLICIT INPUT PARAMETERS:
//
//	A macro with the name of the generic class name followed by the word
//	"declare".  (e.g. vectordeclare)
//
//  EXPANSION VALUE:
//
//	Invocation of the user-defined macro to declare the class.
//	(See implicit input parameters.)
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Declares the class (and possibly functions) which are declared by the
//	user-defined macro.
//
//  DESIGN:
//
//	A sub-macro is used to allow parameter expansion to work.
//
//--

#define declare2(CLASS, T1, T2) cxxl_declare2(CLASS, T1, T2)
#define cxxl_declare2(CLASS, T1, T2) name2(CLASS, declare)(T1, T2)

//++
//  MACRO NAME:	implement2
//
//  FUNCTIONAL DESCRIPTION:
//
//	Implement a generic (parameterized) class for 2 given types.
//	This defines the non-inline member functions of the class.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T1, T2:
//	    The type parameters to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//
//  IMPLICIT INPUT PARAMETERS:
//
//	A macro with the name of the generic class name followed by the word
//	"implement".  (e.g. vectorimplement)
//
//  EXPANSION VALUE:
//
//	Invocation of the user-defined macro to define the non-inline
//	member functions of the class.
//	(See implicit input parameters.)
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Defines the functions which are defined by the user-defined macro.
//
//  DESIGN:
//
//	A sub-macro is used to allow parameter expansion to work.
//
//--

#define implement2(CLASS, T1, T2) cxxl_implement2(CLASS, T1, T2)
#define cxxl_implement2(CLASS, T1, T2) name2(CLASS, implement)(T1, T2)

//++
//  MACRO NAME:	set_handler
//
//  FUNCTIONAL DESCRIPTION:
//
//	Set the current error handler for a given instance of a parameterized
//	class to be the specified function.  (The initial one was
//	genericerror(int, char*).)  In addition to being used as a function
//	call, this macro can also be used in a function declaration or
//	definition.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T:
//	    The type parameters to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//	GPT HANDLER:
//	    A pointer to the new error handler function.
//
//  EXPANSION VALUE:
//
//	The name of the function to set the pointer to the error handler (for a
//	specific instance of a parameterized class) followed by a parenthesized
//	actual argument/formal argument type (e.g. set_intvector_handler(f)).
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	Subsequent calls to the callerror macro will invoke the specified
//	function.
//
//--

#define set_handler(CLASS, T, HANDLER) name4(set_, T, CLASS, _handler)(HANDLER)

//++
//  MACRO NAME: errorhandler
//
//  FUNCTIONAL DESCRIPTION:
//
//	The name of the pointer to the error handler (for a specific instance
//	of a parameterized class).
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T:
//	    The type parameters to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//
//  EXPANSION VALUE:
//
//	The name of the pointer to the error handler (for a specific instance
//	of a parameterized class) (e.g. intvectorhandler).
//
//--

#define errorhandler(CLASS, T) name3(T, CLASS, handler)

//++
//  MACRO NAME: callerror
//
//  FUNCTIONAL DESCRIPTION:
//
//	Call the current error handler for a given instance of a parameterized
//	class.
//
//  FORMAL PARAMETERS:
//
//	CLASS:
//	    The generic class name.
//	    (e.g. vector)
//	T:
//	    The type parameters to instantiate the generic class for.
//	    (e.g. int to get a vector of integers)
//	    Note that the type must be an identifier (e.g. char* is invalid).
//	int N:
//	    The first argument to pass to genericerror(int, char*)
//	    or other, current generic error handling function.
//	char *S:
//	    The second argument to pass to genericerror(int, char*)
//	    or other, current generic error handling function.
//
//  IMPLICIT INPUT PARAMETERS:
//
//	errorhandler(CLASS, T):
//	    Points to the error handling function to call.
//
//  EXPANSION VALUE:
//
//	A call to the function pointed to by errorhandler(CLASS, T)
//	with the given arguments.
//
//--

#define callerror(CLASS, T, N, S) (*errorhandler(CLASS, T))(N, S)


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif


#endif
// End of generic.hxx 
