#ifndef __STRSTREAM_HXX
#define __STRSTREAM_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//==============================================================
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	The iostream package.
//
//  MODULE:	Include file for the strstream subpackage.
//
//  MODULE DESCRIPTION:
//
//      The strstream subpackage provides the "incore" I/O functionality of the
//      iostream package.  It relies on the iostream (core) subpackage's base
//      classes to derive classes that are specialized for I/O performed on
//	arrays of bytes (strings).
//
//--------------------------------------------------------------


//++
//  INCLUDE FILES
//--

#include <compnent.hxx>
#ifdef __USE_STD_IOSTREAM 
#error "cannot include strstream.hxx -- do not compile strict_ansi OR define __NO_USE_STD_IOSTREAM - see section 7.1.2 of the C++ Using Guide"
#else

#include <cxxl.hxx>
#include <iostream.hxx>

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//++
//  LINKAGE
//--

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model  __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif



//++
//  FUNCTIONAL DESCRIPTION:
//
//      A strstreambuf is streambuf that uses an array of bytes (a string) to
//      hold the sequence of characters.  Given the convention that a char *
//      should be interpreted as pointing just before the char it really points
//      at, the mapping between the abstract get/put pointers (see comments for
//      streambuf) and char * pointers is direct.  Moving the pointers
//      corresponds exactly to incrementing and decrementing the char * values.
//
//      To accommodate the need for arbitrary length strings, strstreambuf
//      supports a dynamic mode.  When a strstreambuf is in dynamic mode, space
//      for the character sequence is allocated as needed.  When the sequence
//      is extended too far, it will be copied to a new array.
//--

class strstreambuf : public streambuf
{
public:
			strstreambuf();
			strstreambuf(int);
			strstreambuf(char *, int, char *);
			strstreambuf(unsigned char *, int, unsigned char *);
			strstreambuf(void *(*a)(long), void (*f)(void *));
			~strstreambuf();

    void		freeze(int n = 1);
    virtual int		overflow(int);
    virtual streampos	seekoff(streamoff, __SCOPE(ios::)seek_dir,
				int = ios::in | ios::out);
    virtual streambuf	*setbuf(char *, int);
    char		*str();
    virtual int		underflow();
#if defined(__osf__)
    virtual int		doallocate();
#endif

private:
    void		initialize(char *, int, char *);
    void		initialize_empty();
    int                 __GrowBuffer(int cchRequest);
    void		*(*m_pfnAllocate)(long);
    void                 (*m_pfnFree)(void*);

// Data member definitions -- each platform has own backward compatible layout
#if defined(__VMS) && defined(__VAX)
private:
    int                 m_cchAllocate;     // The size to request
    int                 *m_pExpand;
    char                m_bIsDynamic;
    char                m_bIsFrozen;
    char                m_bIsWriteProtected;
    char                m_bFill1;
#endif

#if defined(__VMS) && defined(__ALPHA)
private:
    int			m_cchAllocate;          // requested size
    int			m_cchDidAllocate;       // allocated size
    long		m_bpGetMark;
    char		m_bIsDynamic;
    char		m_bIsFrozen;
    char		m_bIsWriteProtected;
    char		m_bFill1;
#endif

#if defined(__osf__) || defined(__linux__)
private:
    char                m_bIsDynamic;
    char                m_bIsFrozen;
    char                m_bIsWriteProtected;
    char                fill1;
    int                 m_cchAllocate;     // The size to request
    int                 m_cchDidAllocate;  // The size actually allocated
    int                 m_bpGetMark;
#endif

#if defined(_NT)
private:
    int                 m_cchAllocate;     // The size to request
    int                 m_cchDidAllocate;  // The size actually allocated
    long                m_bpGetMark;
    int                 m_bIsDynamic;
    int                 m_bIsFrozen;
    int                 m_bIsWriteProtected;
#endif
};

#if defined(__osf__)
// Class strstreambase provides a mechanism to include a single strstreambuf
// in all strstream classes.  It saves a memory allocation when
// creating a strstream object, but it complicates initialization.
// Its primary purpose is compatibility
// with previous versions of the C++ class library provided on DEC OSF/1

class strstreambase : virtual public ios {
public:
                        strstreambase();
                        strstreambase(char *, int, char *);
                        ~strstreambase();
    strstreambuf	*rdbuf();
private:
    strstreambuf        m_strstreambuf;
};
#endif

#if defined(__osf__)
class istrstream : public strstreambase, public istream
#else
class istrstream : public istream
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//      istrstream is an istream specialized for incore operations.
//--

{
public:
			istrstream(char *);
			istrstream(char *, int);
			//
			// Note that any const char * passed must have valid 
			// input otherwise putback() may be called which may 
			// cause an attempt to write into read-only data 
			// sections.
			//
			inline istrstream(const char *);
			inline istrstream(const char *, int);
			~istrstream();
    strstreambuf	*rdbuf();
};

//
// Inline istrstream methods:
//
inline istrstream::istrstream( const char* pchBuffer) : istream()
#if defined(__osf__)
#if defined(__DECCXX) && defined(__STD_CFRONT) && !defined(__CXXCFIX4670)
    , strstreambase((char *)pchBuffer, strlen(pchBuffer), 0)
#else
    , strstreambase((char *)pchBuffer, strlen(pchBuffer), NULL)
#endif
{}
#else
{
    ios::init(new strstreambuf((char *)pchBuffer, strlen(pchBuffer), NULL));
}
#endif

inline istrstream::istrstream(const char* pchBuffer, int cchBuffer) : istream()
#if defined(__osf__)
#if defined(__DECCXX) && defined(__STD_CFRONT) && !defined(__CXXCFIX4670)
    , strstreambase((char *) pchBuffer, cchBuffer, 0)
#else
    , strstreambase((char *) pchBuffer, cchBuffer, NULL)
#endif
{}
#else
{
    ios::init(new strstreambuf((char *)pchBuffer, cchBuffer, NULL));
}
#endif


#if defined(__osf__)
class ostrstream : public strstreambase, public ostream
#else
class ostrstream : public ostream
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//      ostrstream is an ostream specialized for incore operations.
//--

{
public:
			ostrstream();
			ostrstream(char *, int, int = ios::out);
		       ~ostrstream();	

    int			pcount();
    strstreambuf	*rdbuf();
    char		*str();
};

#if defined(__osf__)
class strstream : public strstreambase, public iostream
#else
class strstream : public iostream
#endif

//++
//  FUNCTIONAL DESCRIPTION:
//
//      strstream is an iostream specialized for incore operations.
//--

{
public:
			strstream();
			strstream(char *, int, int);
			~strstream();

    strstreambuf	*rdbuf();
    char		*str();
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif // __USE_STD_IOSTREAM

#endif
// End of strstream.hxx
