#ifndef		__TASK_OBJECT_HXX
#define		__TASK_OBJECT_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	task package
//
//  MODULE:	object
//
//  INFORMATION HIDDEN:
//
//	Representation of an "object" in the task package.
//	Representation of the "remember chain" of tasks to be alerted when an
//	object becomes pending.
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of objects and the inline
//	functions which operate on them.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif


//
//
//  TYPE DEFINITIONS
//
//

class object;

class sched;

class timer;

class task;

typedef int (*PFIO)(int, object *);

// The associated messages are in task_object.cxx
// some of these are implementation-dependent and therefore unused
//
// WARNING: The TASK error numbers are duplicated in cxxl.msf.
// Changes to these should be done in both places!
//
enum task_error_code
{
    cxxl_task_error_code_first = 1,	// same as its successor
    E_OLINK = 1,
    E_ONEXT = 2,
    E_GETEMPTY = 3,
    E_PUTOBJ = 4,
    E_PUTFULL = 5,
    E_BACKOBJ = 6,
    E_BACKFULL = 7,
    E_SETCLOCK = 8,
    E_CLOCKIDLE = 9,
    E_RESTERM = 10,
    E_RESRUN = 11,
    E_NEGTIME = 12,
    E_RESOBJ = 13,
    E_HISTO = 14,
    E_STACK = 15,
    E_STORE = 16,
    E_TASKMODE = 17,
    E_TASKDEL = 18,
    E_TASKPRE = 19,
    E_TIMERDEL = 20,
    E_SCHTIME = 21,
    E_SCHOBJ = 22,
    E_QDEL = 23,
    E_RESULT = 24,
    E_WAIT = 25,
    E_FUNCS = 26,
    E_FRAMES = 27,
    E_REGMASK = 28,
    E_FUDGE_SIZE = 29,
    E_NO_HNDLR = 30,
    E_BADSIG = 31,
    E_LOSTHNDLR = 32,
    E_TASKNAMEOVERRUN = 33,
    cxxl_task_error_code_last = 33	// same as its predecessor
};

// these are powers of 2 so that they can be ored together
enum cxxl_task_print_verbosity
{
    VERBOSE =		1 << 0,	// more information
    CHAIN =		1 << 1,	// info. about each object on chain of all tasks
    STACK =		1 << 2,	// info. about the run-time stack
    CXXL_DEBUGGING =	1 << 3	// internal debugging info.
};

class __cxxl_task_chain_t

//++
//  CLASS DESCRIPTION:
//
//	chain/list of tasks
//--

{
    friend class object;

private:
    __cxxl_task_chain_t *cxxl_next;	// pointer to successor chain element
    task *cxxl_task;			// pointer to task at this chain element

public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    inline __cxxl_task_chain_t();

    void insert_head(task *);
    task *remove_head();
    __cxxl_task_chain_t *search(task *);

    inline int empty();
    inline __cxxl_task_chain_t *chain_successor();
    inline task *chain_task();
};

inline __cxxl_task_chain_t::__cxxl_task_chain_t(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Create an empty chain.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    cxxl_next = (__cxxl_task_chain_t *)NULL;
    cxxl_task = (task *)NULL;
}

inline __cxxl_task_chain_t *__cxxl_task_chain_t::chain_successor(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Get a pointer to the successor chain element to this chain element.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    return cxxl_next;
}

inline task *__cxxl_task_chain_t::chain_task(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Get a pointer to the task at this chain element.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    return cxxl_task;
}

inline int __cxxl_task_chain_t::empty(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Is the chain empty?
//
//  RETURN VALUE:
//
//	non-0: The chain is empty
//	0: The chain is not empty
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    return cxxl_next == (__cxxl_task_chain_t *)NULL;
}

class object

//++
//  CLASS DESCRIPTION:
//
//	object in the task package which can be placed on queues or be
//	scheduled or have scheduled objects wait for it (i.e. everything except
//	histograms and random number generators is derived from this class)
//
//	Changes to this class should also be reflected in object::print().
//
//--

{
    friend class sched;
    friend class task;

protected:
    __cxxl_task_chain_t cxxl_remembered_tasks;	// the "remember chain"

public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

    enum objtype	// object type (returned by virtual function o_type())
    {
	OBJECT,		// class object
	TIMER,		// class timer
	TASK,		// class task
	QHEAD,		// class qhead
	QTAIL,		// class qtail
	INTHANDLER	// class Interrupt_handler
    };

    object *o_next;		// next object in a queue or the "run chain"

    static PFIO __CXXL_XFER_DECL(error_fct);// error handler established by 
					    // the application called by 
					    // object::task_error(int, object *)

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif

    inline object();

    ~object();

    void alert();
    void forget(task *);
    void remember(task *);
    int task_error(int);

    virtual objtype o_type();
    virtual int pending();
    virtual void print(int, int = 0);

    static int task_error(int, object *);
    static task *this_task();
};

inline object::object(
    )

//++
//  FUNCTIONAL DESCRIPTION:
//
//	Create an object.
//
//  IMPLICIT OUTPUT PARAMETERS AND SIDE EFFECTS:
//
//	this
//--

{
    o_next = (object *)NULL;
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_object.hxx
