#ifndef		__TASK_QUEUE_HXX
#define		__TASK_QUEUE_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Tasking -- Queues
//
//  MODULE:	task_queue
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of qheads & qtails and the
//	inline functions which operate on them.
//
//  DESIGN ISSUES:
//
//	The objects in a queue are in a circular list, linked together by
//	object::o_next; the last object in the queue points to the first object
//	in the queue, so that none of them have NULL pointers for o_next.  This
//	allows detection of the error of inserting an element into a queue
//	which is already on another queue (o_next != NULL).
//
//	The qhead and qtail each have their own mode, since different error
//	handling strategies could be used for gets from and puts to the same
//	queue.
//
//	Whenever a qhead or a qtail is created, the other end of the queue is
//	automatically created at that time.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
//  TYPE DEFINITIONS
//
//

class object;

class qhead;
class qtail;

enum qmodetype	// queue head/tail error handling mode
{
    EMODE,	// yield a run-time error if full on enqueue/empty on dequeue
    WMODE,	// wait (suspend task) if full on enqueue/empty on dequeue
    ZMODE	// no-op (return NULL) if full on enqueue/empty on dequeue
};



//
//
//  STATIC DATA
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_DATA_LINKAGE
#endif

// the initial default maximum number of entries in a queue
const int CXXL_QUEUE_MAXSIZE = 10000;

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#endif


class qhead: public object

//++
//  CLASS DESCRIPTION:
//
//	queue head
//
//	Changes to this class should also be reflected in qhead::print().
//
//--

{
    friend class qtail;
    friend void cxxl_task_queue_splice(qhead *, qtail *);

private:
    qtail *cxxl_qhead_qtail;		// pointer to qtail of queue
    qmodetype cxxl_qhead_mode;		// what to do on get when empty
    unsigned int cxxl_qhead_current_size;	// number of objects in queue
    unsigned int cxxl_qhead_max_size;	// maximum number of objects in queue

    qhead(qtail *, qmodetype=WMODE, int=CXXL_QUEUE_MAXSIZE);
    inline void cxxl_qhead_init(qmodetype, unsigned int);
    inline void cxxl_qhead_size_adjust(int);

public:
    qhead(qmodetype=WMODE, int=CXXL_QUEUE_MAXSIZE);
    ~qhead();
    qhead *cut();
    object *get();
    int pending();
    int putback(object *);
    int rdcount();
    int rdmax();
    qmodetype rdmode();
    void setmode(qmodetype);
    void setmax(int);
    void splice(qtail *);
    qtail *tail();

    virtual void print(int, int=0);
    virtual objtype o_type();
};

class qtail: public object

//++
//  CLASS DESCRIPTION:
//
//	queue tail
//
//	Changes to this class should also be reflected in qtail::print().
//
//--

{
    friend class qhead;
    friend void cxxl_task_queue_splice(qhead *, qtail *);

private:
    object *cxxl_qtail_last;		// pointer to last object in queue
    qhead *cxxl_qtail_qhead;		// pointer to qhead of queue
    qmodetype cxxl_qtail_mode;		// what to do on put/putback when full

    inline void cxxl_qtail_init(qmodetype);
    qtail(qhead *, qmodetype=WMODE);

public:
    qtail(qmodetype=WMODE, int=CXXL_QUEUE_MAXSIZE);
    ~qtail();
    qtail *cut();
    qhead *head();
    int pending();
    int put(object *);
    int rdspace();
    int rdmax();
    qmodetype rdmode();
    void setmode(qmodetype);
    void setmax(int);
    void splice(qhead *);

    virtual void print(int, int=0);
    virtual objtype o_type();
};


#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of task_queue.hxx
