#ifndef		__VECTOR_HXX
#define		__VECTOR_HXX

//
//    
//  Copyright 2000 Compaq Computer Corporation
//
//  COMPAQ Registered in U.S. Patent and Trademark Office.
//
//  Confidential computer software. Valid license from Compaq required for
//  possession, use or copying. Consistent with FAR 12.211 and 12.212,
//  Commercial Computer Software, Computer Software Documentation, and
//  Technical Data for Commercial Items are licensed to the U.S. Government
//  under vendor's standard commercial license.
//
//

//++
//  FACILITY:	DEC C++ Class Library
//
//  SUBSYSTEM:	Vectors & Stacks
//
//  MODULE:	vector
//
//  INFORMATION HIDDEN:
//
//	Representation of a vector and stack.
//
//  MODULE DESCRIPTION:
//
//	This module implements the representation of vectors & stacks and
//	defines the functions which operate on them.
//
//  DESIGN ISSUES:
//
//	This package uses generic types; see generic.h for details.
//
//	This file contains everything which DOES depend on the parameters
//	of the generic type.
//
//--



//
//
//  INCLUDE FILES
//
//

#include <cxxl.hxx>
#include <generic.hxx>

#ifdef _NT
#include <objectio.hxx>
#else
#include <objection.hxx>
#endif

#include <iostream.hxx>
#include <messages.hxx>

extern "C"
{
#include <stddef.h>
#include <stdlib.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
//  LINKAGE
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __strict_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif


//++
//  CLASS:	vector(TYPE)
//
//  CLASS DESCRIPTION:
//
//	Generic (parameterized) type for a vector of objects of some type.
//
//	The following operations must be defined for the type of the objects:
//	    new
//	    delete
//	    =
//
//  PRIVATE DATA:
//
//	unsigned cxxl_size:
//	    The number of elements in the vector
//	TYPE *cxxl_elements:
//	    The address of the elements
//
//  DESIGN:
//
//	The elements are allocated in a single, contiguous vector.
//	The address of the 0th element is stored in cxxl_elements.
//	The address of the Nth element (N >= 0 && N < cxxl_size) is
//	cxxl_elements((N*sizeof(TYPE)) == cxxl_elements[N].
//
//	The associated messages are in cxxl.msf and cxxl$msg.msg.
//
//	WARNING: The error numbers are duplicated in cxxl.msf.
//	Changes to these should be done in both places!
//--

#define vector(TYPE) name2(TYPE, vector)

#define vectordeclare(TYPE) \
    class vector(TYPE) \
    { \
    private: \
	unsigned cxxl_size; \
	TYPE *cxxl_elements; \
    protected: \
	static Messages messages; \
    public: \
	vector(TYPE)(int); \
	vector(TYPE)(vector(TYPE) &); \
	inline ~vector(TYPE)(); \
	vector(TYPE) &operator=(vector(TYPE) &); \
	inline TYPE &elem(int); \
	inline TYPE &operator[](int); \
	inline int size(); \
	void set_size(int); \
	static Objection size_error; \
	static Objection copy_size_error; \
	static Objection index_error; \
    }; \
    CXXL_VECTOR_DESTRUCTOR(TYPE) \
    CXXL_VECTOR_SIZE(TYPE) \
    CXXL_VECTOR_ELEM(TYPE) \
    CXXL_VECTOR_INDEX(TYPE)

// workaround for CXXC_BUGS 5347
#if defined(__VMS) && defined(__DECCXX)
#pragma extern_prefix save
#pragma extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
# endif

extern Objection_action cxxl_vector_error;

#if defined(__VMS) && defined(__DECCXX)
#pragma extern_prefix restore
#endif

#define vectorimplement(TYPE) \
    CXXL_VECTOR_CONSTRUCTOR(TYPE) \
    CXXL_VECTOR_CONSTRUCTOR_VECTOR(TYPE) \
    CXXL_VECTOR_ASSIGN(TYPE) \
    CXXL_VECTOR_SET_SIZE(TYPE) \
    Objection vector(TYPE)::size_error(&cxxl_vector_error); \
    Objection vector(TYPE)::copy_size_error(&cxxl_vector_error); \
    Objection vector(TYPE)::index_error(&cxxl_vector_error); \
    Messages vector(TYPE)::messages(__cxxl_packages_t::file_name, \
	__cxxl_packages_t::set_vector, __cxxl_packages_t::file_location);

// The error codes used in the message file (for both stack and vector)
enum cxxl_vector_error_code_t
{
    E_VECTOR_SIZE	= 1,
    E_VECTOR_SIZEASSIGN	= 2,
    E_VECTOR_INDEX	= 3,
    E_STACK_UNDERFLOW	= 4,
    E_STACK_OVERFLOW	= 5,
    E_STACK_NOTOP	= 6
};

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Constructor for a generic vector.
//
//  FORMAL PARAMETERS:
//
//	int new_size:
//	    Number of elements in the vector
//
//  EXCEPTIONS:
//
//	size_error
//
//  PRECONDITIONS:
//
//	new_size >= 1
//
//  POSTCONDITIONS:
//
//	size() == new_size
//--

#define CXXL_VECTOR_CONSTRUCTOR(TYPE) \
    vector(TYPE)::vector(TYPE)(int new_size) \
    { \
	if(new_size <= 0) \
	{ \
	    size_error.raise(messages.text(E_VECTOR_SIZE, \
		"Vector size is not at least 1")); \
	    new_size = 1; \
	} \
	cxxl_size = new_size; \
	cxxl_elements = new TYPE[cxxl_size]; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Constructor for a generic vector.
//	The initial values of the elements are taken from another object of
//	the same type and size.
//
//  FORMAL PARAMETERS:
//
//	vector(TYPE) &src:
//	    The vector to copy from
//
//  PRECONDITIONS:
//
//	src.size() == this->size()
//
//  POSTCONDITIONS:
//
//	for(int i=0; i<src.size(); i++) src[i] == (*this)[i]
//--

#define CXXL_VECTOR_CONSTRUCTOR_VECTOR(TYPE) \
    vector(TYPE)::vector(TYPE)(vector(TYPE) &src) \
    { \
	cxxl_size = src.size(); \
	cxxl_elements = new TYPE[cxxl_size]; \
	const int new_size = cxxl_size; \
	for(int i = 0; i < new_size; i++) \
	    cxxl_elements[i] = src[i]; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Destructor for a generic vector.
//--

#define CXXL_VECTOR_DESTRUCTOR(TYPE) \
    inline vector(TYPE)::~vector(TYPE)() \
    { \
	delete[__NOSCOPE(cxxl_size)] cxxl_elements; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Assignment operator for a generic vector.
//
//  FORMAL PARAMETERS:
//
//	vector(TYPE) &src:
//	    The vector from which data is to be copied
//
//  RETURN VALUE:
//
//	*this
//
//  EXCEPTIONS:
//
//	copy_size_error
//
//  PRECONDITIONS:
//
//	src.size() == size()
//
//  POSTCONDITIONS:
//
//	src.size() == size()
//	for(int i=0; i<src.size(); i++) src[i] == (*this)[i]
//--

#define CXXL_VECTOR_ASSIGN(TYPE) \
    vector(TYPE) &vector(TYPE)::operator=(vector(TYPE) &src) \
    { \
	int dst_size = size(); \
	if(dst_size != src.size()) \
	{ \
	    copy_size_error.raise(messages.text(E_VECTOR_SIZEASSIGN, \
		"Mismatching sizes in vector assignment")); \
	    if(dst_size > src.size()) \
		dst_size = src.size(); \
	} \
	for(int i = 0; i < dst_size; i++) \
	    cxxl_elements[i] = src[i]; \
	return *this; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Change the size of a generic vector.
//
//  FORMAL PARAMETERS:
//
//	int new_size:
//	    The new size of the vector.
//
//  EXCEPTIONS:
//
//	size_error
//
//  PRECONDITIONS:
//
//	new_size >= 1
//
//  POSTCONDITIONS:
//
//	size() == new_size
//
//  DESIGN:
//
//	Allocate a new vector of elements, copy the old elements
//	(being careful not to copy more than will fit), and deallocate
//	the old vector of elements.
//--

#define CXXL_VECTOR_SET_SIZE(TYPE) \
    void vector(TYPE)::set_size(int new_size) \
    { \
	if(new_size <= 0) \
	{ \
	    size_error.raise(messages.text(E_VECTOR_SIZE, \
		"Vector size is not at least 1")); \
	    new_size = 1; \
	} \
	int copy_size = size(); \
	if(new_size < copy_size) copy_size = new_size; \
	TYPE *new_elements = new TYPE[new_size]; \
	for(int i = 0; i < copy_size; i++) \
	    new_elements[i] = elem(i); \
	delete[__NOSCOPE(cxxl_size)] cxxl_elements; \
	cxxl_elements = new_elements; \
	cxxl_size = new_size; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Return the number of elements in a generic vector.
//
//  RETURN VALUE:
//
//	The number of elements in the vector
//
//  POSTCONDITIONS:
//
//	return value >= 1
//--

#define CXXL_VECTOR_SIZE(TYPE) \
    inline int vector(TYPE)::size() \
    { \
	return cxxl_size; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Return a reference to an element in a generic vector.
//
//  FORMAL PARAMETERS:
//
//	int i:
//	    An index into the vector
//
//  RETURN VALUE:
//
//	The contents of the ith element of the vector.
//
//  DESIGN:
//
//	Use the default, built-in indexing mechanism.
//--

#define CXXL_VECTOR_ELEM(TYPE) \
    inline TYPE &vector(TYPE)::elem(int i) \
    { \
	return cxxl_elements[i]; \
    }

//++
//  CLASS:	vector(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Index operator for a generic vector.
//
//  FORMAL PARAMETERS:
//
//	int i:
//	    An index into the vector
//
//  RETURN VALUE:
//
//	The contents of the ith element of the vector.
//
//  EXCEPTIONS:
//
//	index_error
//
//  PRECONDITIONS:
//
//	i >= 0 && i < size()
//
//  DESIGN:
//
//	Use the elem(int) member function to do the actual indexing.
//--

#define CXXL_VECTOR_INDEX(TYPE) \
    inline TYPE &vector(TYPE)::operator[](int i) \
    { \
	if((i < 0) || (i >= size())) \
	    index_error.raise(messages.text(E_VECTOR_INDEX, \
		"Vector index out of bounds")); \
	return elem(i); \
    }

//++
//  CLASS:	stack(TYPE)
//
//  BASE CLASSES:	public vector(TYPE)
//
//  CLASS DESCRIPTION:
//
//	Generic (parameterized) type for a stack of objects of some type.
//
//	The following operations must be defined for the type of the objects:
//	    new
//	    delete
//	    =
//
//  PRIVATE DATA:
//
//	unsigned cxxl_n_used:
//	    The number of elements in the stack which are occupied
//
//  DESIGN:
//
//	A stack is a special case (hence a derived class or subclass) of a
//	vector.
//	The number of elements which are currently in the stack is cxxl_n_used.
//	If the stack is empty, cxxl_n_used == 0.
//	If the stack has N elements in it, cxxl_n_used == N and the element is
//	stored in elem(N-1).
//	If the stack is full, cxxl_n_used == size().
//--

#define stack(TYPE) name2(TYPE, stack)

#define stackdeclare(TYPE) \
    class stack(TYPE): protected vector(TYPE) \
    { \
    private: \
	unsigned cxxl_n_used; \
    public: \
	stack(TYPE)(int); \
	stack(TYPE)(stack(TYPE) &); \
	inline void push(TYPE); \
	inline TYPE pop(); \
	inline TYPE &top(); \
	inline int full(); \
	inline int empty(); \
	inline int size(); \
	inline int size_used(); \
	static Objection overflow_error; \
	static Objection underflow_error; \
	static Objection no_top_error; \
    }; \
    CXXL_STACK_PUSH(TYPE) \
    CXXL_STACK_POP(TYPE) \
    CXXL_STACK_TOP(TYPE) \
    CXXL_STACK_FULL(TYPE) \
    CXXL_STACK_EMPTY(TYPE) \
    CXXL_STACK_SIZE(TYPE) \
    CXXL_STACK_SIZE_USED(TYPE)

#define stackimplement(TYPE) \
    CXXL_STACK_CONSTRUCTOR(TYPE) \
    CXXL_STACK_CONSTRUCTOR_STACK(TYPE) \
    Objection stack(TYPE)::overflow_error(&cxxl_vector_error); \
    Objection stack(TYPE)::underflow_error(&cxxl_vector_error); \
    Objection stack(TYPE)::no_top_error(&cxxl_vector_error);

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Constructor for a generic stack.
//
//  FORMAL PARAMETERS:
//
//	int new_size:
//	    Maximum number of elements in the stack
//
//  PRECONDITIONS:
//
//	new_size >= 1
//
//  POSTCONDITIONS:
//
//	size() == new_size
//	size_used() == 0
//
//  DESIGN:
//
//	Use the base-class (vector) constructor to create the data area.
//	Then initialize our private data.
//--

#define CXXL_STACK_CONSTRUCTOR(TYPE) \
    inline stack(TYPE)::stack(TYPE)(int size) \
	: vector(TYPE)(size) \
    { \
	cxxl_n_used = 0; \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Constructor for a generic stack.
//
//  FORMAL PARAMETERS:
//
//	stack(TYPE) &src:
//	    The stack to copy from
//
//  POSTCONDITIONS:
//
//	size() == src.size()
//	size_used() == src.size_used()
//
//  DESIGN:
//
//	Use the base-class (vector) constructor to create (copy) the data area.
//	Then copy our private data.
//--

#define CXXL_STACK_CONSTRUCTOR_STACK(TYPE) \
    inline stack(TYPE)::stack(TYPE)(stack(TYPE) &src) \
	: vector(TYPE)((vector(TYPE) &)(src)) \
    { \
	cxxl_n_used = src.cxxl_n_used; \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Push an element onto a generic stack.
//
//  FORMAL PARAMETERS:
//
//	TYPE new_elem:
//	    The value to push onto the stack
//
//  EXCEPTIONS:
//
//	overflow_error
//
//  PRECONDITIONS:
//
//	size_used() < size()
//
//  POSTCONDITIONS:
//
//	size_used() > 0
//--

#define CXXL_STACK_PUSH(TYPE) \
    inline void stack(TYPE)::push(TYPE new_elem) \
    { \
	if(cxxl_n_used >= size()) \
	    overflow_error.raise(messages.text(E_STACK_OVERFLOW, \
		"Stack overflow")); \
	elem(cxxl_n_used++) = new_elem; \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Pop an element off of the top of a generic stack.
//
//  RETURN VALUE:
//
//	The value popped off of the top of the stack
//
//  EXCEPTIONS:
//
//	underflow_error
//
//  PRECONDITIONS:
//
//	size_used() > 0
//
//  POSTCONDITIONS:
//
//	size_used() < size()
//--

#define CXXL_STACK_POP(TYPE) \
    inline TYPE stack(TYPE)::pop() \
    { \
	if(cxxl_n_used == 0) \
	    underflow_error.raise(messages.text(E_STACK_UNDERFLOW, \
		"Stack underflow")); \
	return elem(--cxxl_n_used); \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Return a reference to the element on the top of a generic stack.
//
//  RETURN VALUE:
//
//	A reference to the value on the top of the stack
//
//  EXCEPTIONS:
//
//	no_top_error
//
//  PRECONDITIONS:
//
//	size_used() > 0
//
//  POSTCONDITIONS:
//
//	size_used() > 0
//--

#define CXXL_STACK_TOP(TYPE) \
    inline TYPE &stack(TYPE)::top() \
    { \
	if(cxxl_n_used == 0) \
	    no_top_error.raise(messages.text(E_STACK_NOTOP, \
		"No element on top of stack")); \
	return elem(cxxl_n_used - 1); \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	See if a generic stack is full.
//
//  RETURN VALUE:
//
//	non-0: There are 0 elements in the stack which are not used
//	    (The stack is full)
//	0: There are >0 elements in the stack which are not used
//	    (The stack is not full)
//--

#define CXXL_STACK_FULL(TYPE) \
    inline int stack(TYPE)::full() \
    { \
	return cxxl_n_used == size(); \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	See if a generic stack is empty.
//
//  RETURN VALUE:
//
//	non-0: There are 0 elements in the stack which are used
//	    (The stack is empty)
//	0: There are >0 elements in the stack which are used
//	    (The stack is not empty)
//--

#define CXXL_STACK_EMPTY(TYPE) \
    inline int stack(TYPE)::empty() \
    { \
	return cxxl_n_used == 0; \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Return the maximum number of elements in a generic stack.
//
//  RETURN VALUE:
//
//	The maximum number of elements which can be occupied in the stack
//--

#define CXXL_STACK_SIZE(TYPE) \
    inline int stack(TYPE)::size() \
    { \
	return vector(TYPE)::size(); \
    }

//++
//  CLASS:	stack(TYPE)
//
//  FUNCTIONAL DESCRIPTION:
//
//	Return the number of elements currently in use in a generic stack.
//
//  RETURN VALUE:
//
//	The number of elements which are currently occupied in the stack
//--

#define CXXL_STACK_SIZE_USED(TYPE) \
    inline int stack(TYPE)::size_used() \
    { \
	return cxxl_n_used; \
    }

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
// End of vector.hxx
